==> ssft-0.10.tar.gz: ssft-0.10/server.cpp <==
/* 
    ssft - is a simple file transfer program that use SSL (Secure 
    Socket Layer) encryption to secure the transfers - and 
    signed certificates to authenticate users and servers. 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Copyright 2001 by Jarle (jgaa) Aase, jgaa@jgaa.com
    http://www.jgaa.com

*/

#include "StdAfx.h"
#include "ssft.h"

using namespace std;

int do_server()
{
    ssft_svr_conn_list_t client_list;
    SOCKET listening_sck;
    sockaddr_in my_addr;

    if (!hostname.empty() && !resolve_hostname())
        return -1;

    // Load sertificate
    SSL_load_error_strings();
    SSLeay_add_ssl_algorithms();
    SSL_METHOD *meth = SSLv23_server_method();
    SSL_CTX *ctx = SSL_CTX_new (meth);
    if (!ctx) 
    {
        SslError ssl_err;

        log << LOG_DATE  
            << "ERROR - SSL_CTX_new() failed " 
            << ssl_err << ENDL;
        
        SSL_CTX_free (ctx);
        return -2;
    }

    if (SSL_CTX_load_verify_locations(ctx, rootcert_file.c_str(), NULL) <= 0)
    {
        SslError ssl_err;

        log << LOG_DATE  
            << "ERROR - SSL_CTX_load_verify_locations(\""
            << rootcert_file
            << "\") failed "
            << ssl_err << ENDL;
        
        SSL_CTX_free (ctx);
        return -3;
    }

    if (SSL_CTX_set_default_verify_paths(ctx) <= 0)
    {
        SslError ssl_err;

        log << LOG_DATE  
            << "ERROR - SSL_CTX_load_verify_locations() failed " 
            << ssl_err << ENDL;
        
        SSL_CTX_free (ctx);
        return -4; 
    }
    
    if (SSL_CTX_use_certificate_file(ctx, cert_file.c_str(), SSL_FILETYPE_PEM) <= 0) 
    {
        SslError ssl_err;

        log << LOG_DATE  
            << "ERROR - SSL_CTX_use_certificate_file() failed " 
            << ssl_err << ENDL;
        
        SSL_CTX_free (ctx);
        return -5;
    }
    
    if (SSL_CTX_use_PrivateKey_file(ctx, key_file.c_str(), SSL_FILETYPE_PEM) <= 0) 
    {
        SslError ssl_err;
        
        log << LOG_DATE  
            << "ERROR - SSL_CTX_use_certificate_file() failed " 
            << ssl_err << ENDL;
        
        return -6;
    }
    
    if (SSL_CTX_check_private_key(ctx) <= 0) 
    {
        SslError ssl_err;

        log << LOG_DATE 
            << "ERROR - Private key does not match the certificate public key " 
            << ENDL;
        
        SSL_CTX_free (ctx);
        return -7;
    }
    
    if (0 == nocert_flag)
    {
        SSL_CTX_set_verify(ctx, SSL_VERIFY_PEER 
            | SSL_VERIFY_FAIL_IF_NO_PEER_CERT | SSL_VERIFY_CLIENT_ONCE,
            verify_callback);
    }

    if ((listening_sck = create_socket(true)) == -1)
    {
        log << LOG_DATE << "Failed to create listening socket." << ENDL;
        if (1024 > my_port)
        {
            log << LOG_DATE
                << "Using port "
                << my_port
                << ". Do you have access to use privilegied ports?"
                << ENDL;
        }
        return -1;
    }
    my_addr = *pin;
    set_async_mode(listening_sck, true);

    if (verbose_flag)
    {
        log << LOG_DATE
            << "Listening for connections at "
            << my_addr
            << ENDL;
    }

    if (listen(listening_sck, 10) != 0)
    {
        log << LOG_DATE << "Failed to listen to server socket! " << EXPLAIN_SOCKET_ERROR << ENDL;
        SSL_CTX_free (ctx);
        return -8;
    }

    log << LOG_DATE << "Ready, waiting for client connection..." << ENDL;
    
    while(do_continue)
    {   
        struct timeval tv;
        tv.tv_sec = 3;
        tv.tv_usec = 0;

        fd_set rd_set, wr_set;
        FD_ZERO(&rd_set);
        FD_ZERO(&wr_set);

        // Assign listening socket as first socket
        SOCKET high_sck = listening_sck;
        FD_SET(listening_sck, &rd_set);

        // Assign clients
        {
            for(ssft_svr_conn_list_t::iterator P = client_list.begin()
                ; P != client_list.end()
                ; P++)
            {
                SOCKET mr_sck = (*P)->GetSocket();
                FD_SET(mr_sck, &rd_set);
                FD_SET(mr_sck, &wr_set);
                high_sck = max(mr_sck, high_sck);
            }
        }
        ++high_sck;

        int result = select(high_sck, &rd_set, &wr_set, NULL, &tv);
        if (result > 0)
        {
            // Check clients
again:
            for(ssft_svr_conn_list_t::iterator P = client_list.begin()
                ; P != client_list.end()
                ; P++)
            {
                if (FD_ISSET((*P)->GetSocket(), &rd_set))
                {
                    try
                    {
                        (*P)->OnRead();
                    }
                    catch(SsftDisconnect& ex)
                    {
                        if (debug_flag)
                        {
                            log << LOG_DATE << "Client from " 
                                << (*P)->Explain().c_str()
                                << " disconnected in SvrConnection::OnRead() "
                                << ex.what()
                                << ENDL;
                        }
kill_connection:
                        delete (*P);
                        client_list.erase(P);
                        goto again;
                    }

                    catch(...)
                    {
                        log << LOG_DATE  
                            << "Caught untrapped exception from SvrConnection::OnRead()" 
                            << ENDL;
                    }
                }

                if (FD_ISSET((*P)->GetSocket(), &wr_set))
                {
                    try
                    {
                        (*P)->OnWrite();
                    }
                    catch(SsftWouldBlock)
                    {
                        ; // Ignore
                    }
                    catch(SsftDisconnect& ex)
                    {
                        if (debug_flag)
                        {
                            log << LOG_DATE << "Client from " 
                                << (*P)->Explain().c_str()
                                << " disconnected in SvrConnection::OnWrite() "
                                << ex.what()
                                << ENDL;
                        }

                        goto kill_connection;
                    }
                    catch(...)
                    {
                        log << LOG_DATE  
                            << "Caught untrapped exception from SvrConnection::OnWrite()" 
                            << ENDL;
                    }
                }
            }

            // Check listening socket
            if (FD_ISSET(listening_sck, &rd_set))
            {
                // Listening socket
                socklen_t sockaddr_len = sizeof(sockaddr);
                SOCKET conn = accept(listening_sck, &socaddr, &sockaddr_len);
                if (-1 == conn)
                {
                    log << LOG_DATE << "Connection failed. " << GET_SOCKET_ERROR << ENDL;
                    continue;
                }
                
                log << LOG_DATE  
                    << "Client connected from " 
                    << *pin
                    << ENDL;
                
                BOOL opt = true;
                setsockopt(conn, IPPROTO_TCP, TCP_NODELAY, (const char *)&opt, sizeof(BOOL));
                set_async_mode(conn, true);
                
                SvrConnection *pconn = new SvrConnection(conn, ctx, pin);
                pconn->Create();
                
                client_list.push_back(pconn);
            }
        }
    }

    SSL_CTX_free (ctx);
    clear_user_list();
    
    return 0;
}

int verify_callback(int preverify_ok, X509_STORE_CTX *ctx)
{
    char    buf[256];
    X509   *err_cert;
    int     err, depth;
    SSL    *ssl;
    
    err_cert = X509_STORE_CTX_get_current_cert(ctx);
    err = X509_STORE_CTX_get_error(ctx);
    depth = X509_STORE_CTX_get_error_depth(ctx);
    
    ssl = (SSL *)X509_STORE_CTX_get_ex_data(ctx, SSL_get_ex_data_X509_STORE_CTX_idx());
    
    X509_NAME_oneline(X509_get_subject_name(err_cert), buf, 256);

    if (!preverify_ok) 
    {
        log << LOG_DATE
            << "Client certificate not accepted: verify error: num="
            << err
            << ' '
            << X509_verify_cert_error_string(err)
            << " depth="
            << depth
            << ' '
            << buf
            << ENDL;
    }
    else if (debug_flag)
    {
        log << LOG_DATE
            << "Client certificate OK: depth="
            << depth
            << ' '
            << buf
            << ENDL;
    }
    
    if (!preverify_ok 
        && (err == X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT))
    {
        X509_NAME_oneline(X509_get_issuer_name(ctx->current_cert), buf, 256);
        log << LOG_DATE
            << "Client certificate: issuer="
            << buf
            << ENDL;
    }
            
    return preverify_ok;
    //return 0;
}

==> ssft-0.12.tar.gz: ssft-0.12/server.cpp <==
/* 
    ssft - is a simple file transfer program that use SSL (Secure 
    Socket Layer) encryption to secure the transfers - and 
    signed certificates to authenticate users and servers. 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Copyright 2001 by Jarle (jgaa) Aase, jgaa@jgaa.com
    http://www.jgaa.com

*/

#include "StdAfx.h"
#include "ssft.h"

using namespace std;

int do_server()
{
    ssft_svr_conn_list_t client_list;
    SOCKET listening_sck;
    sockaddr_in my_addr;

    if (!hostname.empty() && !resolve_hostname())
        return -1;

    // Load sertificate
    SSL_load_error_strings();
    SSLeay_add_ssl_algorithms();
    SSL_METHOD *meth = SSLv23_server_method();
    SSL_CTX *ctx = SSL_CTX_new (meth);
    if (!ctx) 
    {
        SslError ssl_err;

        log << LOG_DATE  
            << "ERROR - SSL_CTX_new() failed " 
            << ssl_err << ENDL;
        
        SSL_CTX_free (ctx);
        return -2;
    }

    if (SSL_CTX_load_verify_locations(ctx, rootcert_file.c_str(), NULL) <= 0)
    {
        SslError ssl_err;

        log << LOG_DATE  
            << "ERROR - SSL_CTX_load_verify_locations(\""
            << rootcert_file
            << "\") failed "
            << ssl_err << ENDL;
        
        SSL_CTX_free (ctx);
        return -3;
    }

    if (SSL_CTX_set_default_verify_paths(ctx) <= 0)
    {
        SslError ssl_err;

        log << LOG_DATE  
            << "ERROR - SSL_CTX_load_verify_locations() failed " 
            << ssl_err << ENDL;
        
        SSL_CTX_free (ctx);
        return -4; 
    }
    
    if (SSL_CTX_use_certificate_file(ctx, cert_file.c_str(), SSL_FILETYPE_PEM) <= 0) 
    {
        SslError ssl_err;

        log << LOG_DATE  
            << "ERROR - SSL_CTX_use_certificate_file() failed " 
            << ssl_err << ENDL;
        
        SSL_CTX_free (ctx);
        return -5;
    }
    
    if (SSL_CTX_use_PrivateKey_file(ctx, key_file.c_str(), SSL_FILETYPE_PEM) <= 0) 
    {
        SslError ssl_err;
        
        log << LOG_DATE  
            << "ERROR - SSL_CTX_use_certificate_file() failed " 
            << ssl_err << ENDL;
        
        return -6;
    }
    
    if (SSL_CTX_check_private_key(ctx) <= 0) 
    {
        SslError ssl_err;

        log << LOG_DATE 
            << "ERROR - Private key does not match the certificate public key " 
            << ENDL;
        
        SSL_CTX_free (ctx);
        return -7;
    }
    
    if (0 == nocert_flag)
    {
        SSL_CTX_set_verify(ctx, SSL_VERIFY_PEER 
            | SSL_VERIFY_FAIL_IF_NO_PEER_CERT | SSL_VERIFY_CLIENT_ONCE,
            verify_callback);
    }

    if ((listening_sck = create_socket(true)) == -1)
    {
        log << LOG_DATE << "Failed to create listening socket." << ENDL;
        if (1024 > my_port)
        {
            log << LOG_DATE
                << "Using port "
                << my_port
                << ". Do you have access to use privilegied ports?"
                << ENDL;
        }
        return -1;
    }
    my_addr = *pin;
    set_async_mode(listening_sck, true);

    if (verbose_flag)
    {
        log << LOG_DATE
            << "Listening for connections at "
            << my_addr
            << ENDL;
    }

    if (listen(listening_sck, 10) != 0)
    {
        log << LOG_DATE << "Failed to listen to server socket! " << EXPLAIN_SOCKET_ERROR << ENDL;
        SSL_CTX_free (ctx);
        return -8;
    }

    log << LOG_DATE << "Ready, waiting for client connection..." << ENDL;
    
    while(do_continue)
    {   
        struct timeval tv;
        tv.tv_sec = 3;
        tv.tv_usec = 0;

        fd_set rd_set, wr_set;
        FD_ZERO(&rd_set);
        FD_ZERO(&wr_set);

        // Assign listening socket as first socket
        SOCKET high_sck = listening_sck;
        FD_SET(listening_sck, &rd_set);

        // Assign clients
        {
            for(ssft_svr_conn_list_t::iterator P = client_list.begin()
                ; P != client_list.end()
                ; P++)
            {
                SOCKET mr_sck = (*P)->GetSocket();
                FD_SET(mr_sck, &rd_set);
                FD_SET(mr_sck, &wr_set);
                high_sck = max(mr_sck, high_sck);
            }
        }
        ++high_sck;

        int result = select(high_sck, &rd_set, &wr_set, NULL, &tv);
        if (result > 0)
        {
            // Check clients
again:
            for(ssft_svr_conn_list_t::iterator P = client_list.begin()
                ; P != client_list.end()
                ; P++)
            {
                if (FD_ISSET((*P)->GetSocket(), &rd_set))
                {
                    try
                    {
                        (*P)->OnRead();
                    }
                    catch(SsftDisconnect& ex)
                    {
                        if (debug_flag)
                        {
                            log << LOG_DATE << "Client from " 
                                << (*P)->Explain().c_str()
                                << " disconnected in SvrConnection::OnRead() "
                                << ex.what()
                                << ENDL;
                        }
kill_connection:
                        delete (*P);
                        client_list.erase(P);
                        goto again;
                    }

                    catch(...)
                    {
                        log << LOG_DATE  
                            << "Caught untrapped exception from SvrConnection::OnRead()" 
                            << ENDL;
                    }
                }

                if (FD_ISSET((*P)->GetSocket(), &wr_set))
                {
                    try
                    {
                        (*P)->OnWrite();
                    }
                    catch(SsftWouldBlock)
                    {
                        ; // Ignore
                    }
                    catch(SsftDisconnect& ex)
                    {
                        if (debug_flag)
                        {
                            log << LOG_DATE << "Client from " 
                                << (*P)->Explain().c_str()
                                << " disconnected in SvrConnection::OnWrite() "
                                << ex.what()
                                << ENDL;
                        }

                        goto kill_connection;
                    }
                    catch(...)
                    {
                        log << LOG_DATE  
                            << "Caught untrapped exception from SvrConnection::OnWrite()" 
                            << ENDL;
                    }
                }
            }

            // Check listening socket
            if (FD_ISSET(listening_sck, &rd_set))
            {
                // Listening socket
                socklen_t sockaddr_len = sizeof(sockaddr);
                SOCKET conn = accept(listening_sck, &socaddr, &sockaddr_len);
                if (-1 == conn)
                {
                    log << LOG_DATE << "Connection failed. " << GET_SOCKET_ERROR << ENDL;
                    continue;
                }
                
                log << LOG_DATE  
                    << "Client connected from " 
                    << *pin
                    << ENDL;
                
                BOOL opt = true;
                //setsockopt(conn, IPPROTO_TCP, TCP_NODELAY, (const char *)&opt, sizeof(BOOL));
                set_async_mode(conn, true);
                
                SvrConnection *pconn = new SvrConnection(conn, ctx, pin);
                pconn->Create();
                
                client_list.push_back(pconn);
            }
        }
    }

    SSL_CTX_free (ctx);
    clear_user_list();
    
    return 0;
}

int verify_callback(int preverify_ok, X509_STORE_CTX *ctx)
{
    char    buf[256];
    X509   *err_cert;
    int     err, depth;
    SSL    *ssl;
    
    err_cert = X509_STORE_CTX_get_current_cert(ctx);
    err = X509_STORE_CTX_get_error(ctx);
    depth = X509_STORE_CTX_get_error_depth(ctx);
    
    ssl = (SSL *)X509_STORE_CTX_get_ex_data(ctx, SSL_get_ex_data_X509_STORE_CTX_idx());
    
    X509_NAME_oneline(X509_get_subject_name(err_cert), buf, 256);

    if (!preverify_ok) 
    {
        log << LOG_DATE
            << "Client certificate not accepted: verify error: num="
            << err
            << ' '
            << X509_verify_cert_error_string(err)
            << " depth="
            << depth
            << ' '
            << buf
            << ENDL;
    }
    else if (debug_flag)
    {
        log << LOG_DATE
            << "Client certificate OK: depth="
            << depth
            << ' '
            << buf
            << ENDL;
    }
    
    if (!preverify_ok 
        && (err == X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT))
    {
        X509_NAME_oneline(X509_get_issuer_name(ctx->current_cert), buf, 256);
        log << LOG_DATE
            << "Client certificate: issuer="
            << buf
            << ENDL;
    }
            
    return preverify_ok;
    //return 0;
}
